# Behat Statistic Extension

Easy way to store behat build statistic in database

## Configuration

```yaml
default: &default
    extensions: &default_extensions
        Oro\Bundle\TestFrameworkBundle\BehatStatisticExtension\ServiceContainer\BehatStatisticExtension:
            connection:
                dbname: dev_behat_stats
                user: dev
                password: 123456
                host: localhost
                driver: pdo_mysql
            criteria:
                branch_name: CHANGE_BRANCH
                target_branch: CHANGE_TARGET
                build_id: BUILD_ID
                single_branch_name: BRANCH_NAME
                any_var: ANY_ENV_VAR # this could be used in AvgTimeProvider
            count_build_limit: 10
```

To see all existing configuration connections abilities, follow Doctrine Dbal documentation -
http://docs.doctrine-project.org/projects/doctrine-dbal/en/latest/reference/configuration.html

```CHANGE_BRANCH```, ```CHANGE_TARGET```, ```BRANCH_NAME``` and ```BUILD_ID```
is required environment variables names for build in providers.
However any AvgTimeProviders could be used, see ```AvgTimeProviderInterface```

## Usage

Use ```-f statistic``` formatter argument to enable store statistic to database:
```bash
bin/behat -f statistic
```

Use ```--suite-divider``` option to specify how much features should be in one autogenerated suite.
Usually you would specify 1 to get divide more accuracy

Use ```--max_suite_set_execution_time``` option for specify max execution time in seconds for one suite set.
Duration of each suite will be count by average time of features regardless to criteria env variables.

Use ```--dump-suite-sets``` and ```--dump-suites``` options to commit dividing to files.
Suites and suite sets can be imported while execution
through ```BEHAT_SUITE_SETS``` and ```BEHAT_SUITES``` environment variables.
It can be valid json as string or path to json file with data.

So the flow is following:

1. Make sure that you have sufficient statistics in your database
2. Make dividing on master node:
```bash
bin/behat \
    --suite-divider=1 \
    --max_suite_set_execution_time=360 \
    --available-suite-sets \
    --dump-suite-sets="./suite-sets.json" \
    --dump-suites="./suites.json" \

```
3. Create nodes for each suite set and run each set on each node
 and pass suite set name somehow (e.g. through SUITE_SET env var):
```bash
BEHAT_SUITE_SETS="./suite-sets.json"
BEHAT_SUITES="./suites.json"
bin/behat --suite-set="$SUITE_SET"
```


For more see the [behat features](./Features) including this package

### Tests

To run behat tests for extension install dev dependencies and run:

```bash
bin/behat
bin/phpunit
```

### Behat suite registry

This extension replace ```Behat\Testwork\Suite\Cli\SuiteController``` and
add new object ```SuiteConfigurationRegistry``` because list of suites
generates according to parameters ```--suite-divider```, ```--suite-set-divider``` and ```--max_suite_set_execution_time```


